--------------------------------------------------------------------------------
-- Broker_ProfessionsMenu v2.0                                                --
-- Author: Pathur                                                             --
-- Changed: 16.07.10                                                          --
--------------------------------------------------------------------------------



--------------------------------------------------------------------------------
-- Register Events, Init Variables                                            --
--------------------------------------------------------------------------------
local _, me = ...                                 --Includes all functions and variables

me.frame = CreateFrame("FRAME")
me.frame:RegisterEvent("ADDON_LOADED")
me.frame:RegisterEvent("PLAYER_LOGIN")
me.frame:RegisterEvent("PLAYER_LOGOUT")
me.frame:RegisterEvent("PLAYER_REGEN_DISABLED")
me.frame:RegisterEvent("PLAYER_REGEN_ENABLED")
me.frame:RegisterEvent("TRADE_SKILL_UPDATE")

me.dewdrop = LibStub('Dewdrop-2.0', true)      --Dropdownmenu

me.combat = false                                 --In fight
me.launcher = {}                                  --Quick Launcher
me.sharedcds = {}                                 --shared cds

if me.P.SharedCDs then
 for k, v in pairs(me.P.SharedCDs) do
  if GetSpellInfo(k) then                         --check if spell exists
   me.sharedcds[GetSpellInfo(k)] = v
  end
 end
end



--------------------------------------------------------------------------------
-- GET SKILLS, WEAPON SKILLS, RANKS, ICONS                                    --
--------------------------------------------------------------------------------
--Localize Profession / Include Icons / Set Secure or Disabled
function me:LocalizeProf(name, id)
  local spell = {}
  spell.name,_,spell.icon,_,_,_,_,_,_ = GetSpellInfo(id)
   if (strfind(name,",Secure")~=nil) then
    spell.secure = true
   elseif (strfind(name,",Disabled")~=nil) then
    spell.disabled = true
   end
  spell.id = id
  return spell
end
--Localize List of all Professions in idarray and Creat Lines
function me:LocalizeList(profarray)               --Professions ID-Array from professions.lua
  local k, v, kk, vv
  local result = {}
  local spell = {}
  for k, v in pairs(profarray) do
   if (type(v)=="number") then
    result = me:LocalizeProf(k,v)
    spell[result.name] = result
   else
    result = me:LocalizeProf(k,v["*"])
    spell[result.name] = result
    local help = result.name
    for kk, vv in pairs(profarray[k]) do
     if (kk~="*") then
      result = me:LocalizeProf(kk,vv)
      if (spell[help]["more"]==nil) then spell[help]["more"] = {} end
      spell[help]["more"][result.name] = result
     end
    end
   end
  end
  return spell
end
--Set New Table
me.P.Primary = me:LocalizeList(me.P.Primary)
me.P.Secondary = me:LocalizeList(me.P.Secondary)
--Filter professions and add Ranks
function me:FilterProfs(profarray)                --Array from me:LocalizeList()
  local spell = {}
  for i = 1, GetNumSkillLines() do
   local skillName, isHeader, _, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
   if (isHeader == nil) then
    if (profarray[skillName]~=nil) then
     spell[skillName] = profarray[skillName]
     spell[skillName]["name"] = nil
     spell[skillName]["rank"] = skillRank
     spell[skillName]["maxrank"] = skillMaxRank
    end
   else
    ExpandSkillHeader(i)
   end
  end
  return spell	
end
--Get Weapon Skills
function me:GetWeaponSkills()
  local skills = {}
  local active = false
  for i = 1, GetNumSkillLines() do
   local skillName, isHeader, _, skillRank, _, _, skillMaxRank = GetSkillLineInfo(i)
   if (isHeader==1) then
    if (skillName==me.L["weapon"]) then
     active=true
     ExpandSkillHeader(i)
    else
     active=false
    end
   else
    if (active == true) then
     skills[skillName] = {}
     skills[skillName]["rank"] = skillRank
     skills[skillName]["maxrank"] = skillMaxRank
    end
   end 
  end
  return skills
end



--------------------------------------------------------------------------------
--Table Alphabetical Sort Function                                            --
--------------------------------------------------------------------------------
function me:pairsByKeys (t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
   table.sort(a, f)
   local i = 0                                    --iterator variable
   local iter = function ()                       --iterator function
   i = i + 1
   if a[i] == nil then
    return nil
   else
    return a[i], t[a[i]]
   end
  end
  return iter
end



--------------------------------------------------------------------------------
--Format Cooldown Duration                                                    --
--------------------------------------------------------------------------------
function me:formatduration(duration)
  local d,h,m
  d = floor(duration/86400)
  h = floor(duration/3600)-(d*24)
  m = floor(duration/60)-((d*1440)+(h*60))
  if d==0 then
   if h==0 then
    if m==0 then
     return "<1"..me.L["m"]
    else
     return m..me.L["m"]
    end
   else
    return h..me.L["h"].." "..m..me.L["m"]
   end
  else
   return d..me.L["d"].." "..h..me.L["h"]
  end 
end



--------------------------------------------------------------------------------
-- tcount: count table members even if they're not indexed by numbers         --
--------------------------------------------------------------------------------
function me:tcount(tab)
  local n = #tab
  if (n == 0) then
   for _ in pairs(tab) do
    n = n + 1
   end
  end
  return n
 end



--------------------------------------------------------------------------------
-- Load/Save Saved Variables
--------------------------------------------------------------------------------
function me.frame:OnEvent(event, arg1)
  if event == "ADDON_LOADED" and arg1 == "Broker_ProfessionsMenu" then
  ---Default values
   if (ProfessionsMenu == nil) then ProfessionsMenu = {} end
   if (ProfessionsMenu[GetCVar("realmName")] == nil) then ProfessionsMenu[GetCVar("realmName")] = {} end
   if (ProfessionsMenu[GetCVar("realmName")][UnitName("player")] == nil) then ProfessionsMenu[GetCVar("realmName")][UnitName("player")] = {} end
   --Copy saves to internal var and initializise
   me.save = ProfessionsMenu[GetCVar("realmName")][UnitName("player")]
   me.saveallchars = ProfessionsMenu[GetCVar("realmName")]  --all Chars from same server
   if (me.save.config == nil) then
    me.save.config = {
     showprimary = true,
     showsecondary = true,
     showother = true,
     showcds = true,
    }
   end
   if (me.save.default == nil) then
    me.save.default = {
     left = "---",
     shiftleft = "---",
     altleft = "---",
     ctrlleft = "---",
     icon = "Interface\\Icons\\Trade_BlackSmithing"
    }
   end
   if (me.save.cds == nil) then me.save.cds = {} end
   if (me.save.tradelinks == nil) then me.save.tradelinks = {} end
   if (me.save.launcher == nil) then me.save.launcher = {} end
  elseif event == "PLAYER_LOGIN" then
          --Init LibDataBroker
          me:InitLDB(me.save.default.left,me.save.default.icon)
          --Create Launchers 
          for k,v in pairs(me.save.launcher) do
           me.launcher[k]=me:newlauncher(k,v)
          end
	elseif event == "PLAYER_LOGOUT" then
	       me.save.faction = UnitFactionGroup("player")
	       ProfessionsMenu[GetCVar("realmName")] = me.saveallchars
          ProfessionsMenu[GetCVar("realmName")][UnitName("player")] = me.save
	elseif event == "PLAYER_REGEN_DISABLED" then
	       me.combat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
	       me.combat = false
	elseif event == "TRADE_SKILL_UPDATE" then
	       if not IsTradeSkillLinked() and GetNumTradeSkills()~=nil then
		     if GetTradeSkillListLink()~=nil then
		      me.save.tradelinks[strmatch(GetTradeSkillListLink(),"|h%[([%a%s]+)%]|h")] = GetTradeSkillListLink() 
		      --Remove deleted Profession
	         local tradelinks = me.save.tradelinks
		      me.save.tradelinks = {}
	         for k,v in pairs(me:FilterProfs(me.P.Primary)) do
             if tradelinks[k]~=nil then
              me.save.tradelinks[k] = tradelinks[k]
             end
            end   
            for k,v in pairs(me:FilterProfs(me.P.Secondary)) do
             if tradelinks[k]~=nil then
              me.save.tradelinks[k] = tradelinks[k]
             end
            end
           --
           end
	        for i = 1, GetNumTradeSkills() do
	         if skilltype~="header" then
	          local name = GetTradeSkillInfo(i)
                  local cd=GetTradeSkillCooldown(i)
                  if cd then
                   if me.sharedcds[name] then name=me.sharedcds[name] end
                   me.save.cds[name]=time()+floor(cd+0.5)
                  end
	         end
	        end 
	       end
	end
end
me.frame:SetScript("OnEvent", me.frame.OnEvent)   --Register Events



--------------------------------------------------------------------------------
-- Init LibDataBroker (Main Window)
--------------------------------------------------------------------------------
function me:InitLDB(txt,icon)
  --Create Secure Frame Overlay
  me.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate");
  me.secureframe:SetScript("OnLeave",function(self)
   if not me.combat then self:Hide() end
   GameTooltip:Hide()
  end)
  me.secureframe:SetScript("OnEnter",function(self) me:tooltip(self); GameTooltip:Show() end)
  me.secureframe:SetScript("OnMouseDown",function(self,button)
   me.secureframe:SetAttribute("*type1", "spell")
   if IsControlKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.ctrlleft)
   elseif IsAltKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.altleft)
   elseif IsShiftKeyDown() then
    me.secureframe:SetAttribute("spell", me.save.default.shiftleft)
   else
    me.secureframe:SetAttribute("spell", me.save.default.left)
   end
   if button == "RightButton" then
    GameTooltip:Hide()
    me.dewdrop:Open(self, 'children', function(level, value) me:UpdateMenu(level, value) end)
   end
  end)
  --Create LDB
  me.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Broker_ProfessionsMenu", {
   type = "data source",
   text = txt,
   icon = icon,
   OnClick = function(self,btn)
    if me.combat then
     me:Error(me.L["infight"])
     return
    end
    if btn == "RightButton" then
     me.moveable = false
     GameTooltip:Hide()
    end
   end,
   OnEnter = function(self)
    if me.combat then
     return
    end
    if me.moveable then
     me:moveabletooltip(self)
     GameTooltip:Show()
     return
    end
    me.secureframe:SetFrameStrata(self:GetFrameStrata())
    me.secureframe:SetFrameLevel(self:GetFrameLevel()+1)
    me.secureframe:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",0,0)
    me.secureframe:SetWidth(self:GetWidth())
    me.secureframe:SetHeight(self:GetHeight()) 
    me.secureframe:Show()
   end
  })
end



--------------------------------------------------------------------------------
-- Refresh broker icon/text
--------------------------------------------------------------------------------
function me:Professions_UpdateInfo(txt,icon)
  me.dataobj.text = txt
  me.dataobj.icon = icon
end



--------------------------------------------------------------------------------
-- Error Messages / Other Messages
--------------------------------------------------------------------------------
function me:Error(txt)
  UIErrorsFrame:AddMessage(txt,1.0,0,0,100,3);
end



--------------------------------------------------------------------------------
-- Creates Launchers
--------------------------------------------------------------------------------
function me:newlauncher(spell,icon)
  local launcher = {}
  --secure frame
  launcher.secureframe = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")
  launcher.secureframe:SetScript("OnLeave",function(self)
   if not me.combat then self:Hide() end
   GameTooltip:Hide()
  end)
  launcher.secureframe:SetScript("OnEnter",function(self)
   me:quicklaunchertooltip(self,spell)
   GameTooltip:Show()
  end)
  launcher.secureframe:SetAttribute("type1", "spell")
  launcher.secureframe:SetAttribute("spell", spell)
  --Create Dataobject
  launcher.dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Launcher: "..spell, {
   type = "data source",
   text = spell,
   icon = icon,
   OnClick = function(self,btn)
    if me.combat then
     me:Error(me.L["infight"])
     return
    end
    if btn == "RightButton" then
     me.moveable = false
     GameTooltip:Hide()
    end
   end,
   OnEnter = function(self)
    if me.combat then
     return
    end    
    if me.moveable then
     me:moveabletooltip(self)
     GameTooltip:Show()
     return 
    end
    launcher.secureframe:SetFrameStrata(self:GetFrameStrata())
    launcher.secureframe:SetFrameLevel(self:GetFrameLevel()+1)
    launcher.secureframe:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT",0,0)
    launcher.secureframe:SetWidth(self:GetWidth())
    launcher.secureframe:SetHeight(self:GetHeight()) 
    launcher.secureframe:Show()
   end
  })
  return launcher
end